@@ -133,7 +133,7 @@ class FONA:
         self._uart.reset_input_buffer()
 
         self.uart_write(b"ATI\r\n")
-        self._read_line(multiline=True)
+        self.read_line(multiline=True)
 
         if self._buf.find(b"SIM808 R14") != -1:
             self._fona_type = FONA_808_V2
@@ -147,7 +147,7 @@ class FONA:
         if self._fona_type == FONA_800_L:
             # determine if SIM800H
             self.uart_write(b"AT+GMM\r\n")
-            self._read_line(multiline=True)
+            self.read_line(multiline=True)
 
             if self._buf.find(b"SIM800H") != -1:
                 self._fona_type = FONA_800_H
@@ -177,20 +177,8 @@ class FONA:
     @property
     # pylint: disable=too-many-return-statements
     def version(self):
-        """Returns FONA Version, as a string."""
-        if self._fona_type == FONA_800_L:
-            return "FONA 800L"
-        if self._fona_type == FONA_800_H:
-            return "FONA 800H"
-        if self._fona_type == FONA_808_V1:
-            return "FONA 808 (v1)"
-        if self._fona_type == FONA_808_V2:
-            return "FONA 808 (v2)"
-        if self._fona_type == FONA_3G_A:
-            return "FONA 3G (US)"
-        if self._fona_type == FONA_3G_E:
-            return "FONA 3G (EU)"
-        return -1
+        """Returns FONA Version."""
+        return self._fona_type
 
     @property
     def iemi(self):
@@ -200,7 +188,7 @@ class FONA:
         self._uart.reset_input_buffer()
 
         self.uart_write(b"AT+GSN\r\n")
-        self._read_line(multiline=True)
+        self.read_line(multiline=True)
         iemi = self._buf[0:15]
         return iemi.decode("utf-8")
 
@@ -208,7 +196,7 @@ class FONA:
     def local_ip(self):
         """Returns the local IP Address, False if not set."""
         self.uart_write(b"AT+CIFSR\r\n")
-        self._read_line()
+        self.read_line()
         try:
             ip_addr = self.pretty_ip(self._buf)
         except ValueError:
@@ -221,9 +209,9 @@ class FONA:
         if self._debug:
             print("ICCID")
         self.uart_write(b"AT+CCID\r\n")
-        self._read_line(timeout=2000)  # 6.2.23, 2sec max. response time
+        self.read_line(timeout=2000)  # 6.2.23, 2sec max. response time
         iccid = self._buf.decode()
-        self._read_line()  # eat 'OK'
+        self.read_line()  # eat 'OK'
         return iccid
 
     @property
@@ -252,7 +240,7 @@ class FONA:
             # enable multi connection mode (3,1)
             if not self._send_check_reply(b"AT+CIPMUX=1", reply=REPLY_OK):
                 return False
-            self._read_line()
+            self.read_line()
 
             # enable receive data manually (7,2)
             if not self._send_check_reply(b"AT+CIPRXGET=1", reply=REPLY_OK):
@@ -360,7 +348,7 @@ class FONA:
             rssi = map_range(reply_num, 2, 30, -110, -54)
 
         # read out the 'ok'
-        self._read_line()
+        self.read_line()
         return rssi
 
     @property
@@ -379,7 +367,7 @@ class FONA:
             status = int(self._buf[10:11].decode("utf-8"))
             if status == 1:
                 status = 3  # assume 3D fix
-            self._read_line()
+            self.read_line()
         elif self._fona_type == FONA_3G_A or self._fona_type == FONA_3G_E:
             raise NotImplementedError(
                 "FONA 3G not currently supported by this library."
@@ -408,14 +396,14 @@ class FONA:
         if self._fona_type == FONA_808_V2:
             if not self._send_parse_reply(b"AT+CGPSPWR?", b"+CGPSPWR: ", ":"):
                 return False
-        self._read_line()
+        self.read_line()
         if not self._send_parse_reply(b"AT+CGNSPWR?", b"+CGNSPWR: ", ":"):
             return False
 
         state = self._buf
 
         if gps_on and not state:
-            self._read_line()
+            self.read_line()
             if self._fona_type == FONA_808_V2:
                 # try GNS
                 if not self._send_check_reply(b"AT+CGNSPWR=1", reply=REPLY_OK):
@@ -450,9 +438,9 @@ class FONA:
             return False
 
         # attempt to parse a response
-        self._read_line()
+        self.read_line()
         while not self._parse_reply(b"+CDNSGIP:", idx=2):
-            self._read_line()
+            self.read_line()
 
         return self._buf
 
@@ -467,6 +455,24 @@ class FONA:
 
     ### SMS ###
 
+    @property
+    def enable_sms_notification(self):
+        """Returns status of new SMS message notifications"""
+        if not self._send_parse_reply(b"AT+CNMI?\r\n", b"+CNMI:", idx=1):
+            return False
+        return self._buf
+
+    @enable_sms_notification.setter
+    def enable_sms_notification(self, enable=True):
+        """Enables or disables new SMS message notifications."""
+        if enable:
+            if not self._send_check_reply(b"AT+CNMI=2,1\r\n", reply=REPLY_OK):
+                return False
+        else:
+            if not self._send_check_reply(b"AT+CNMI=2,0\r\n", reply=REPLY_OK):
+                return False
+        return True
+
     def send_sms(self, phone_number, message):
         """Sends a message SMS to a phone number.
         :param int phone_number: Destination phone number.
@@ -481,23 +487,23 @@ class FONA:
             return False
 
         self.uart_write(b'AT+CMGS="+' + str(phone_number).encode() + b'"' + b"\r")
-        self._read_line()
+        self.read_line()
 
         # expect >
         if self._buf[0] != 62:
             # promoting mark ('>') not found
             return False
-        self._read_line()
+        self.read_line()
 
         # write out message and ^z
         self.uart_write((message + chr(26)).encode())
 
         if self._fona_type == FONA_3G_A or self._fona_type == FONA_3G_E:
             # eat 2x CRLF
-            self._read_line(200)
-            self._read_line(200)
+            self.read_line(200)
+            self.read_line(200)
         # read +CMGS, wait ~10sec.
-        self._read_line(10000)
+        self.read_line(10000)
 
         if not "+CMGS" in self._buf:
             return False
@@ -525,18 +531,18 @@ class FONA:
             ):
                 return self._buf
 
-        self._read_line()  # eat OK
+        self.read_line()  # eat OK
         if self._send_parse_reply(b"AT+CPMS?", b'"SM",', idx=1):
             return self._buf
 
-        self._read_line()  # eat OK
+        self.read_line()  # eat OK
         if self._send_parse_reply(b"AT+CPMS?", b'"SM_P",', idx=1):
             return self._buf
         return False
 
     def delete_all_sms(self):
         """Deletes all SMS messages on the FONA SIM."""
-        self._read_line()
+        self.read_line()
         if not self._send_check_reply(b"AT+CMGF=1", reply=REPLY_OK):
             return False
 
@@ -573,7 +579,7 @@ class FONA:
             return False
 
         self.uart_write(b"AT+CMGR=" + str(sms_slot).encode() + b"\r\n")
-        self._read_line(1000)
+        self.read_line(1000)
         resp = self._buf
 
         # get sender
@@ -589,9 +595,11 @@ class FONA:
 
         self._buf = bytearray(sms_len)
         self._uart.readinto(self._buf)
+        message = bytes(self._buf).decode()
         self._uart.reset_input_buffer()
+        self.read_line()  # eat 'OK'
 
-        return sender, bytes(self._buf).decode()
+        return sender, message
 
     ### Socket API (TCP, UDP) ###
 
@@ -603,19 +611,19 @@ class FONA:
             print("*** Get socket")
 
         self.uart_write(b"AT+CIPSTATUS\r\n")
-        self._read_line(100)  # OK
-        self._read_line(100)  # table header
+        self.read_line(100)  # OK
+        self.read_line(100)  # table header
 
         allocated_socket = 0
         for sock in range(0, FONA_MAX_SOCKETS):  # check if INITIAL state
-            self._read_line(100)
+            self.read_line(100)
             self._parse_reply(b"C:", idx=5)
             if self._buf.strip('"') == "INITIAL" or self._buf.strip('"') == "CLOSED":
                 allocated_socket = sock
                 break
         # read out the rest of the responses
         for _ in range(allocated_socket, FONA_MAX_SOCKETS):
-            self._read_line(100)
+            self.read_line(100)
         if self._debug:
             print("Allocated socket #%d" % allocated_socket)
         return allocated_socket
@@ -630,7 +638,7 @@ class FONA:
         ), "Provided socket exceeds the maximum number of \
                                              sockets for the FONA module."
         self.uart_write(b"AT+CIPSTATUS=" + str(sock_num).encode() + b"\r\n")
-        self._read_line(100)
+        self.read_line(100)
 
         self._parse_reply(b"+CIPSTATUS:", idx=3)
         return self._buf
@@ -646,10 +654,10 @@ class FONA:
                                              sockets for the FONA module."
         if not self._send_check_reply(b"AT+CIPSTATUS", reply=REPLY_OK, timeout=100):
             return False
-        self._read_line()
+        self.read_line()
 
         for state in range(0, sock_num + 1):  # read "C: <n>" for each active connection
-            self._read_line()
+            self.read_line()
             if state == sock_num:
                 break
         self._parse_reply(b"C:", idx=5)
@@ -658,7 +666,7 @@ class FONA:
 
         # eat the rest of the sockets
         for _ in range(sock_num, FONA_MAX_SOCKETS):
-            self._read_line()
+            self.read_line()
 
         if not "CONNECTED" in state:
             return False
@@ -683,8 +691,8 @@ class FONA:
         if self._debug:
             print("\t {} bytes available.".format(self._buf))
 
-        self._read_line()
-        self._read_line()
+        self.read_line()
+        self.read_line()
 
         return data
 
@@ -719,7 +727,7 @@ class FONA:
 
         # Query local IP Address
         self.uart_write(b"AT+CIFSR\r\n")
-        self._read_line()
+        self.read_line()
 
         # Start connection
         self.uart_write(b"AT+CIPSTART=" + str(sock_num).encode())
@@ -752,7 +760,7 @@ class FONA:
 
         self.uart_write(b"AT+CIPCLOSE=" + str(sock_num).encode() + b",")
         self.uart_write(str(quick_close).encode() + b"\r\n")
-        self._read_line()
+        self.read_line()
         if not self._parse_reply(b"CLOSE OK", idx=0):
             return False
         return True
@@ -776,7 +784,7 @@ class FONA:
         self.uart_write(b",")
         self.uart_write(str(length).encode() + b"\r\n")
 
-        self._read_line()
+        self.read_line()
 
         if not self._parse_reply(b"+CIPRXGET:"):
             return False
@@ -791,7 +799,7 @@ class FONA:
         :param bytes buffer: Bytes to write to socket.
 
         """
-        self._read_line()
+        self.read_line()
         assert (
             sock_num < FONA_MAX_SOCKETS
         ), "Provided socket exceeds the maximum number of \
@@ -800,14 +808,14 @@ class FONA:
         self._uart.reset_input_buffer()
         self.uart_write(b"AT+CIPSEND=" + str(sock_num).encode())
         self.uart_write(b"," + str(len(buffer)).encode() + b"\r\n")
-        self._read_line()
+        self.read_line()
 
         if self._buf[0] != 62:
             # promoting mark ('>') not found
             return False
 
         self.uart_write(buffer + b"\r\n")
-        self._read_line(3000)
+        self.read_line(3000)
 
         if "SEND OK" not in self._buf.decode():
             return False
@@ -816,6 +824,11 @@ class FONA:
 
     ### UART Reply/Response Helpers ###
 
+    @property
+    def in_waiting(self):
+        """Returns number of bytes available to be read in the input buffer."""
+        return self._uart.in_waiting
+
     def uart_write(self, buffer):
         """UART ``write`` with optional debug that prints
         the buffer before sending.
@@ -833,7 +846,7 @@ class FONA:
         :param str divider: Separator
 
         """
-        self._read_line()
+        self.read_line()
         self._get_reply(send_data)
 
         if not self._parse_reply(reply_data, divider, idx):
@@ -855,7 +868,7 @@ class FONA:
         else:
             self.uart_write(prefix + suffix + b"\r\n")
 
-        return self._read_line(timeout)
+        return self.read_line(timeout)
 
     def _parse_reply(self, reply, divider=",", idx=0):
         """Attempts to find reply in UART buffer, reads up to divider.
@@ -881,7 +894,7 @@ class FONA:
 
         return True
 
-    def _read_line(self, timeout=FONA_DEFAULT_TIMEOUT_MS, multiline=False):
+    def read_line(self, timeout=FONA_DEFAULT_TIMEOUT_MS, multiline=False):
         """Reads one or multiple lines into the buffer. Optionally prints the buffer
         after reading.
         :param int timeout: Time to wait for UART serial to reply, in seconds.
@@ -915,7 +928,7 @@ class FONA:
         if self._debug:
             print("\tUARTREAD ::", self._buf.decode())
 
-        return reply_idx
+        return reply_idx, self._buf
 
     def _send_check_reply(
         self,
@@ -930,7 +943,7 @@ class FONA:
         :param bytes reply: Expected response from module.
 
         """
-        self._read_line()
+        self.read_line()
         if send is None:
             if not self._get_reply(prefix=prefix, suffix=suffix, timeout=timeout):
                 return False
@@ -973,14 +986,14 @@ class FONA:
 
         self.uart_write(prefix + b'"' + suffix + b'"\r\n')
 
-        return self._read_line(timeout)
+        return self.read_line(timeout)
 
     def _expect_reply(self, reply, timeout=10000):
         """Reads line from FONA module and compares to reply from FONA module.
         :param bytes reply: Expected reply from module.
 
         """
-        self._read_line(timeout)
+        self.read_line(timeout)
         if reply not in self._buf:
             return False
         return True
